"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DEPENDENCIES_TYPES = exports.ComponentGraph = void 0;
function _graph() {
  const data = require("@teambit/graph.cleargraph");
  _graph = function () {
    return data;
  };
  return data;
}
function _duplicateDependency() {
  const data = require("../duplicate-dependency");
  _duplicateDependency = function () {
    return data;
  };
  return data;
}
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
const DEPENDENCIES_TYPES = exports.DEPENDENCIES_TYPES = ['dependencies', 'devDependencies'];
class ComponentGraph extends _graph().Graph {
  // component IDs that started the graph. (if from workspace, the .bitmap ids normally)
  constructor(nodes = [], edges = []) {
    super(nodes, edges);
    _defineProperty(this, "seederIds", []);
  }
  create(nodes = [], edges = []) {
    return new ComponentGraph(nodes, edges);
  }

  /**
   * @deprecate use graph.getGraphIds().findCycles()
   */
  findCycles(graph) {
    const cycles = super.findCycles(graph);
    if (!this.shouldLimitToSeedersOnly()) {
      return cycles;
    }
    const seederIdsStr = this.seederIds.map(id => id.toString());
    const cyclesWithSeeders = cycles.filter(cycle => {
      return cycle.some(cycleIdStr => seederIdsStr.includes(cycleIdStr));
    });
    return cyclesWithSeeders;
  }
  findDuplicateDependencies() {
    const versionMap = this.calculateVersionMap();
    const seederIdsNoVersions = this.seederIds.map(id => id.toStringWithoutVersion());
    const duplicateDependencies = new Map();
    for (const [compFullName, versions] of versionMap) {
      if (versions.allVersionNodes.length > 1) {
        const versionSubgraphs = [];
        const notLatestVersions = versions.allVersionNodes.filter(version => version !== versions.latestVersionNode);
        notLatestVersions.forEach(version => {
          const predecessors = this.predecessorsSubgraph(version);
          const immediatePredecessors = this.predecessors(version).map(predecessor => predecessor.id);
          const subGraph = this.buildFromCleargraph(predecessors);
          const versionSubgraph = {
            versionId: version,
            subGraph,
            // TODO: validate that this is working correctly
            immediateDependents: immediatePredecessors
          };
          versionSubgraphs.push(versionSubgraph);
        });
        const isSeeder = seederIdsNoVersions.includes(compFullName);
        const shouldDisplayDueToBeingSeeder = !this.shouldLimitToSeedersOnly() || isSeeder;
        if (shouldDisplayDueToBeingSeeder && versionSubgraphs.length > 0) {
          const duplicateDep = new (_duplicateDependency().DuplicateDependency)(versions.latestVersionNode, versionSubgraphs);
          duplicateDependencies.set(compFullName, duplicateDep);
        }
      }
    }
    return duplicateDependencies;
  }
  buildFromCleargraph(graph) {
    return this.create(graph.nodes, graph.edges);
  }
  runtimeOnly(componentIds) {
    return this.successorsSubgraph(componentIds, {
      edgeFilter: edge => edge.attr.type === 'runtime'
    });
  }
  shouldLimitToSeedersOnly() {
    return this.seederIds.length;
  }
  calculateVersionMap() {
    const versionMap = new Map();
    for (const node of this.nodes) {
      const comp = node.attr;
      const compKey = node.id;
      const compFullName = comp.id.toStringWithoutVersion();
      if (!versionMap.has(compFullName)) {
        versionMap.set(compFullName, {
          allVersionNodes: [compKey],
          latestVersionNode: compKey
        });
      } else {
        const value = versionMap.get(compFullName);
        if (value) {
          if (Object.prototype.hasOwnProperty.call(value, 'allVersionNodes')) {
            value.allVersionNodes.push(compKey);
          }
          const currentComp = comp;
          const latestComp = this.node(value.latestVersionNode)?.attr;
          // @todo: this check won't work when the component doesn't have head.
          // it happens when a dependency is needed in an old version (not head). which Bit doesn't fetch the head
          // Version object, and as a result, the `Component.head` is empty.
          // for now it's probably good enough because it's used only for `findDuplicateDependencies`, which only
          // checks the components on the workspace.
          if (currentComp.head && latestComp?.head && new Date(currentComp.head.timestamp) > new Date(latestComp.head.timestamp)) {
            value.latestVersionNode = compKey;
          }
        }
      }
    }
    return versionMap;
  }
}
exports.ComponentGraph = ComponentGraph;

//# sourceMappingURL=component-graph.js.map