import { ComponentID } from '@teambit/component-id';
import { Graph, Node, Edge } from '@teambit/graph.cleargraph';
import type { DependenciesInfo } from '@teambit/legacy.dependency-graph';
import GraphLib from 'graphlib';
export type DepEdgeType = 'prod' | 'dev' | 'ext' | 'peer';
type ComponentIdNode = Node<ComponentID>;
type DependencyEdge = Edge<DepEdgeType>;
export type CompIdGraph = Graph<ComponentID, DepEdgeType>;
export declare class ComponentIdGraph extends Graph<ComponentID, DepEdgeType> {
    private _graphLib;
    seederIds: ComponentID[];
    constructor(nodes?: ComponentIdNode[], edges?: DependencyEdge[]);
    get graphLib(): GraphLib.Graph;
    protected create(nodes?: ComponentIdNode[], edges?: DependencyEdge[]): this;
    /**
     * check all the routes from the sources to targets and return the components found during this traversal.
     * e.g.
     * A -> B -> C -> N.
     * A -> E -> N.
     * B -> F -> G.
     * given source: A, targets: N. The results will be: [B, C, E].
     *
     * if through is provided, it will only return the components that are connected to the through components.
     * with the example above, if through is B, the results will be: [B, C].
     */
    findIdsFromSourcesToTargets(sources: ComponentID[], targets: ComponentID[], through?: ComponentID[]): ComponentID[];
    /**
     * check all the routes from the sources to targets and return the components found during this traversal.
     * e.g.
     * A -> B -> C -> N.
     * A -> E -> N.
     * B -> F -> G.
     * given source: A, targets: N. The results will be: [B, C, E].
     *
     * if through is provided, it will only return the components that are connected to the through components.
     * with the example above, if through is B, the results will be: [B, C].
     */
    findAllPathsFromSourcesToTargets(sources: ComponentID[], targets: ComponentID[], through?: ComponentID[]): string[][];
    /**
     * overrides the super class to eliminate non-seeders components
     */
    findCycles(graph?: this, includeDeps?: boolean): string[][];
    buildFromCleargraph(graph: Graph<ComponentID, DepEdgeType>): ComponentIdGraph;
    runtimeOnly(componentIds: string[]): this;
    getDependenciesInfo(id: ComponentID): DependenciesInfo[];
    getDependenciesAsObjectTree(idStr: string): Record<string, any>;
    private shouldLimitToSeedersOnly;
}
export {};
