"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GraphBuilder = void 0;
function _graph() {
  const data = require("@teambit/graph.cleargraph");
  _graph = function () {
    return data;
  };
  return data;
}
function _componentGraph() {
  const data = require("./component-graph");
  _componentGraph = function () {
    return data;
  };
  return data;
}
function _componentIdGraph() {
  const data = require("./component-id-graph");
  _componentIdGraph = function () {
    return data;
  };
  return data;
}
function _dependency() {
  const data = require("./model/dependency");
  _dependency = function () {
    return data;
  };
  return data;
}
class GraphBuilder {
  constructor(componentAspect) {
    this.componentAspect = componentAspect;
  }

  /**
   * important - prefer using `getGraphIds()` it's way better in terms of performance.
   */
  async getGraph(ids, opts = {}) {
    const componentHost = opts.host || this.componentAspect.getHost();
    const graph = await componentHost.getGraph(ids, false);
    const componentGraph = await this.toComponentGraph(graph);
    componentGraph.seederIds = ids || (await componentHost.listIds());
    return componentGraph;
  }
  async getGraphIds(ids, opts = {}) {
    const componentHost = opts.host || this.componentAspect.getHost();
    const graph = await componentHost.getGraphIds(ids, false);
    const componentIdGraph = new (_componentIdGraph().ComponentIdGraph)(graph.nodes, graph.edges);
    componentIdGraph.seederIds = ids || (await componentHost.listIds());
    return componentIdGraph;
  }
  async toComponentGraph(graph) {
    const newGraph = new (_componentGraph().ComponentGraph)();
    graph.nodes.forEach(node => {
      newGraph.setNode(node);
    });
    graph.edges.forEach(edge => {
      const edgeObj = edge.attr === 'dependencies' || edge.attr === 'runtime' ? new (_dependency().Dependency)('runtime') : new (_dependency().Dependency)('dev');
      newGraph.setEdge(new (_graph().Edge)(edge.sourceId, edge.targetId, edgeObj));
    });
    return newGraph;
  }
}
exports.GraphBuilder = GraphBuilder;

//# sourceMappingURL=graph-builder.js.map