"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GraphCmd = void 0;
function _chalk() {
  const data = _interopRequireDefault(require("chalk"));
  _chalk = function () {
    return data;
  };
  return data;
}
function _graphlib() {
  const data = _interopRequireDefault(require("graphlib"));
  _graphlib = function () {
    return data;
  };
  return data;
}
function _componentId() {
  const data = require("@teambit/component-id");
  _componentId = function () {
    return data;
  };
  return data;
}
function _legacy() {
  const data = require("@teambit/legacy.dependency-graph");
  _legacy = function () {
    return data;
  };
  return data;
}
function _scope() {
  const data = require("@teambit/scope.remotes");
  _scope = function () {
    return data;
  };
  return data;
}
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
class GraphCmd {
  constructor(componentAspect, graph) {
    this.componentAspect = componentAspect;
    this.graph = graph;
    _defineProperty(this, "name", 'graph [id]');
    _defineProperty(this, "description", "generate an SVG image file with the components' dependencies graph");
    _defineProperty(this, "extendedDescription", void 0);
    _defineProperty(this, "group", 'info-analysis');
    _defineProperty(this, "alias", '');
    _defineProperty(this, "options", [['r', 'remote [remoteName]', 'remote name (name is optional, leave empty when id is specified)'], ['', 'layout <name>', 'GraphVis layout. default to "dot". options are [circo, dot, fdp, neato, osage, patchwork, sfdp, twopi]'], ['', 'png', 'save the graph as a png file instead of svg. requires "graphviz" to be installed'], ['', 'cycles', 'generate a graph of cycles only'], ['', 'include-local-only', 'DEPRECATED: include only the components in the workspace (or local scope). This is now the default behavior.'], ['', 'include-dependencies', 'include all dependencies recursively, not just workspace (or local scope) components'], ['j', 'json', 'json format']]);
    _defineProperty(this, "remoteOp", true);
  }
  async report([id], graphOpts) {
    const {
      remote,
      layout,
      png,
      includeLocalOnly,
      includeDependencies
    } = graphOpts;
    const host = this.componentAspect.getHost();
    const getVisualGraph = async () => {
      if (remote) {
        const config = {};
        if (layout) config.layout = layout;
        const graph = await this.generateGraphFromRemote(remote, id);
        return _legacy().VisualDependencyGraph.loadFromGraphlib(graph, config);
      }
      const compId = id ? await host.resolveComponentId(id) : undefined;
      const compIds = compId ? [compId] : undefined;

      // New logic: local-only is now the default behavior
      // includeDependencies flag overrides this to show all dependencies
      const modifiedOpts = _objectSpread(_objectSpread({}, graphOpts), {}, {
        includeLocalOnly: includeDependencies ? false : includeLocalOnly !== false // true by default unless includeDependencies is used
      });
      return this.graph.getVisualGraphIds(compIds, modifiedOpts);
    };
    const visualDependencyGraph = await getVisualGraph();
    const result = await visualDependencyGraph.render(png ? 'png' : 'svg');
    return _chalk().default.green(`image created at ${result}`);
  }
  async json([id], graphOpts) {
    const {
      remote,
      includeLocalOnly,
      includeDependencies
    } = graphOpts;
    const host = this.componentAspect.getHost();
    if (!remote) {
      // For JSON output, we need to manually filter if needed
      // Since getGraphIds doesn't accept the same options as getVisualGraphIds
      const shouldIncludeLocalOnly = includeDependencies ? false : includeLocalOnly !== false;
      const graph = await this.graph.getGraphIds(id ? [await host.resolveComponentId(id)] : undefined);
      let filteredGraph = graph;
      if (shouldIncludeLocalOnly) {
        // Filter to only include local components
        const list = await host.listIds();
        const listStr = list.map(compId => compId.toString());
        filteredGraph = graph.successorsSubgraph(listStr, {
          nodeFilter: node => listStr.includes(node.id),
          edgeFilter: edge => listStr.includes(edge.targetId) && listStr.includes(edge.sourceId)
        });
      }
      const jsonGraph = filteredGraph.toJson();
      if (jsonGraph.nodes) {
        jsonGraph.nodes = jsonGraph.nodes.map(node => node.id);
      }
      return jsonGraph;
    }
    const graph = await this.generateGraphFromRemote(remote, id);
    return _graphlib().default.json.write(graph);
  }
  getWorkspaceIfExist() {
    try {
      return this.componentAspect.getHost('teambit.workspace/workspace');
    } catch {
      return undefined;
    }
  }
  async generateGraphFromRemote(remote, id) {
    const workspace = this.getWorkspaceIfExist();
    const compId = id ? _componentId().ComponentID.fromString(id) : undefined;
    if (compId) {
      const scopeName = typeof remote === 'string' ? remote : compId.scope;
      const remoteScope = await (0, _scope().getRemoteByName)(scopeName, workspace?.consumer);
      const componentDepGraph = await remoteScope.graph(compId);
      return componentDepGraph.graph;
    }
    if (typeof remote !== 'string') {
      throw new Error('please specify remote scope name or enter an id');
    }
    const remoteScope = await (0, _scope().getRemoteByName)(remote, workspace?.consumer);
    const componentDepGraph = await remoteScope.graph();
    return componentDepGraph.graph;
  }
}
exports.GraphCmd = GraphCmd;

//# sourceMappingURL=graph-cmd.js.map