"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GraphMain = void 0;
function _cli() {
  const data = require("@teambit/cli");
  _cli = function () {
    return data;
  };
  return data;
}
function _component() {
  const data = require("@teambit/component");
  _component = function () {
    return data;
  };
  return data;
}
function _graphql() {
  const data = require("@teambit/graphql");
  _graphql = function () {
    return data;
  };
  return data;
}
function _logger() {
  const data = require("@teambit/logger");
  _logger = function () {
    return data;
  };
  return data;
}
function _lodash() {
  const data = require("lodash");
  _lodash = function () {
    return data;
  };
  return data;
}
function _graphBuilder() {
  const data = require("./graph-builder");
  _graphBuilder = function () {
    return data;
  };
  return data;
}
function _graph() {
  const data = require("./graph.graphql");
  _graph = function () {
    return data;
  };
  return data;
}
function _graph2() {
  const data = require("./graph.aspect");
  _graph2 = function () {
    return data;
  };
  return data;
}
function _graphCmd() {
  const data = require("./graph-cmd");
  _graphCmd = function () {
    return data;
  };
  return data;
}
function _legacy() {
  const data = require("@teambit/legacy.dependency-graph");
  _legacy = function () {
    return data;
  };
  return data;
}
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
class GraphMain {
  constructor(componentAspect, logger) {
    this.componentAspect = componentAspect;
    this.logger = logger;
  }

  /**
   * important - prefer using `getGraphIds()` it's way better in terms of performance.
   */
  async getGraph(ids, opts = {}) {
    const graphBuilder = new (_graphBuilder().GraphBuilder)(this.componentAspect);
    return graphBuilder.getGraph(ids, opts);
  }
  async getGraphIds(ids, opts = {}) {
    const graphBuilder = new (_graphBuilder().GraphBuilder)(this.componentAspect);
    return graphBuilder.getGraphIds(ids, opts);
  }

  /**
   * this visual graph-ids can render the graph as a SVG/png and other formats.
   */
  async getVisualGraphIds(ids, opts = {}) {
    this.logger.setStatusLine('loading graph');
    const {
      layout,
      includeLocalOnly,
      cycles
    } = opts;
    const graphVizOpts = {};
    if (layout) graphVizOpts.layout = layout;
    const graphIdsAll = await this.getGraphIds(ids);
    const host = this.componentAspect.getHost();
    const list = await host.listIds();
    const idsWithVersion = await this.getIdsWithVersions(ids);
    const listStr = list.map(id => id.toString());
    const graphIds = includeLocalOnly ? graphIdsAll.successorsSubgraph(idsWithVersion || listStr, {
      nodeFilter: node => listStr.includes(node.id),
      edgeFilter: edge => listStr.includes(edge.targetId) && listStr.includes(edge.sourceId)
    }) : graphIdsAll;
    this.logger.setStatusLine('rendering graph');
    if (cycles) {
      return this.getVisualCyclesFromGraph(graphIds, idsWithVersion, graphVizOpts);
    }
    return _legacy().VisualDependencyGraph.loadFromClearGraph(graphIds, graphVizOpts, idsWithVersion);
  }
  async getVisualCyclesFromGraph(graphIds, idsWithVersion, graphVizOpts = {}) {
    const cyclesGraph = graphIds.findCycles();
    const multipleCycles = cyclesGraph.map(cycle => {
      if (idsWithVersion && (0, _lodash().intersection)(idsWithVersion, cycle).length < 1) return undefined;
      return graphIds.subgraph(cycle, {
        nodeFilter: node => cycle.includes(node.id),
        edgeFilter: edge => cycle.includes(edge.targetId)
      });
    });
    return _legacy().VisualDependencyGraph.loadFromMultipleClearGraphs((0, _lodash().compact)(multipleCycles), graphVizOpts, idsWithVersion);
  }
  async getIdsWithVersions(ids) {
    const host = this.componentAspect.getHost();
    if (!ids) return undefined;
    const comps = await host.getMany(ids);
    if (comps.length) return comps.map(comp => comp.id.toString());
    return undefined;
  }
  static async provider([graphql, componentAspect, cli, loggerMain]) {
    const logger = loggerMain.createLogger(_graph2().GraphAspect.id);
    const graphBuilder = new (_graphBuilder().GraphBuilder)(componentAspect);
    graphql.register(() => (0, _graph().graphSchema)(graphBuilder, componentAspect));
    const graphMain = new GraphMain(componentAspect, logger);
    cli.register(new (_graphCmd().GraphCmd)(componentAspect, graphMain));
    return graphMain;
  }
}
exports.GraphMain = GraphMain;
_defineProperty(GraphMain, "slots", []);
_defineProperty(GraphMain, "dependencies", [_graphql().GraphqlAspect, _component().ComponentAspect, _cli().CLIAspect, _logger().LoggerAspect]);
_defineProperty(GraphMain, "runtime", _cli().MainRuntime);
_graph2().GraphAspect.addRuntime(GraphMain);

//# sourceMappingURL=graph.main.runtime.js.map