"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.diffGraph = diffGraph;
function _compareGraphModel() {
  const data = require("./compare-graph-model");
  _compareGraphModel = function () {
    return data;
  };
  return data;
}
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == typeof i ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != typeof t || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != typeof i) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
const toShortId = node => node.componentId.toStringWithoutVersion();

// this is to get a key with versions ignored so that we'll have a unique set of component nodes
const toShortIdFromNodeId = nodeId => nodeId.split('@')[0];
const delim = '::';
const getEdgeId = _e => {
  return `${toShortIdFromNodeId(_e.sourceId)}${delim}${toShortIdFromNodeId(_e.targetId)}`;
};
function diffGraph(baseGraph, compareGraph, baseId) {
  if (!baseGraph || !compareGraph || !baseId) return null;
  const baseNodes = baseGraph.nodes;
  const compareNodes = compareGraph.nodes;
  const baseNodesMap = new Map(baseNodes.map(n => [toShortId(n), n]));
  const compareNodesMap = new Map(compareNodes.map(n => [toShortId(n), n]));
  const allNodes = [];
  for (const baseNode of baseNodes) {
    const compareNode = compareNodesMap.get(toShortId(baseNode));
    if (compareNode) {
      allNodes.push(_objectSpread(_objectSpread({}, baseNode), {}, {
        compareVersion: compareNode.componentId.version,
        status: compareNode.componentId.isEqual(baseNode.componentId) ? undefined : 'modified'
      }));
    } else {
      allNodes.push(_objectSpread(_objectSpread({}, baseNode), {}, {
        compareVersion: baseNode.componentId.version,
        status: 'deleted'
      }));
    }
  }
  const newNodes = compareNodes.filter(n => !baseNodesMap.has(toShortId(n)));
  for (const node of newNodes) {
    allNodes.push(_objectSpread(_objectSpread({}, node), {}, {
      compareVersion: '',
      status: 'new'
    }));
  }
  const allNodesMap = new Map(allNodes.map(n => [toShortId(n), n]));
  const baseEdgesMap = new Map(baseGraph.edges.map(baseEdge => [getEdgeId(baseEdge), baseEdge]));
  const edgesOnlyInCompare = compareGraph.edges.filter(compareEdge => !baseEdgesMap.has(getEdgeId(compareEdge))).map(compareEdge => _objectSpread(_objectSpread({}, compareEdge), {}, {
    sourceId: allNodesMap.get(toShortIdFromNodeId(compareEdge.sourceId))?.id.toString() || baseId.toString(),
    targetId: allNodesMap.get(toShortIdFromNodeId(compareEdge.targetId))?.id.toString() || baseId.toString()
  }));
  const allEdges = [...baseGraph.edges, ...edgesOnlyInCompare];
  return new (_compareGraphModel().CompareGraphModel)(allNodes, allEdges);
}

//# sourceMappingURL=diff-graph.js.map