"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useGraphQuery = useGraphQuery;
function _react() {
  const data = require("react");
  _react = function () {
    return data;
  };
  return data;
}
function _client() {
  const data = require("@apollo/client");
  _client = function () {
    return data;
  };
  return data;
}
function _graphql() {
  const data = require("@teambit/graphql");
  _graphql = function () {
    return data;
  };
  return data;
}
function _getGraph() {
  const data = require("./get-graph.query");
  _getGraph = function () {
    return data;
  };
  return data;
}
function _graphModel() {
  const data = require("./graph-model");
  _graphModel = function () {
    return data;
  };
  return data;
}
function useGraphQuery(componentId, filter) {
  // Eagerly fetch GET_GRAPH_IDS
  const {
    data: idsData,
    error: idsError,
    loading: idsLoading
  } = (0, _client().useQuery)(_getGraph().GET_GRAPH_IDS, {
    variables: {
      ids: componentId,
      filter
    },
    skip: !componentId || !filter
  });

  // Lazily fetch GET_GRAPH
  const [getGraph, {
    data: graphData,
    error: graphError,
    loading: graphLoading
  }] = (0, _client().useLazyQuery)(_getGraph().GET_GRAPH);
  const [fetchError, setFetchError] = (0, _react().useState)(undefined);
  const [shouldRefetchGraph, setShouldRefetchGraph] = (0, _react().useState)(false);
  (0, _react().useEffect)(() => {
    if (idsData?.graph.nodes.length || !filter) {
      setShouldRefetchGraph(true);
    }
  }, [idsData?.graph.nodes.length, filter]);
  (0, _react().useEffect)(() => {
    if (shouldRefetchGraph) {
      setShouldRefetchGraph(false);
      void getGraph({
        variables: {
          ids: componentId,
          filter
        }
      }).catch(error => {
        setFetchError(new (_graphql().GraphQlError)(500, error.message));
      });
    }
  }, [componentId, filter, getGraph, shouldRefetchGraph]);
  const rawGraph = idsLoading ? undefined : idsData?.graph && graphData?.graph && idsData?.graph.nodes.length === graphData?.graph.nodes.length && graphData?.graph || idsData?.graph || graphData?.graph;
  const clientError = !rawGraph && !idsLoading && !graphLoading ? new (_graphql().GraphQlError)(404) : undefined;
  const serverError = graphError?.message || idsError?.message ? new (_graphql().GraphQlError)(500, graphError?.message || idsError?.message) : fetchError;
  return (0, _react().useMemo)(() => {
    return {
      graph: rawGraph ? _graphModel().GraphModel.from(rawGraph) : undefined,
      error: serverError || clientError,
      loading: idsLoading || graphLoading,
      idsLoading,
      graphLoading
    };
  }, [rawGraph, serverError, clientError, idsLoading, graphLoading]);
}

//# sourceMappingURL=use-graph-query.js.map